import tensorflow as tf
from sklearn.model_selection import train_test_split
import keras
from keras.datasets import imdb
from keras.models import Sequential
from keras.layers import Dense, LSTM, Embedding
from keras.callbacks import ModelCheckpoint

#Loading and formatting the data
(x_train, y_train), (x_test, y_test) = imdb.load_data(num_words=5000)
x_train = keras.preprocessing.sequence.pad_sequences(x_train,maxlen=500)
x_test = keras.preprocessing.sequence.pad_sequences(x_test,maxlen=500)
x_train,x_val,y_train,y_val = train_test_split(x_train,y_train,test_size=0.2)
print(x_train.shape,x_val.shape,y_train.shape,y_val.shape)

    
#Defining the model with Keras
model = Sequential()
# The Embedding layer here takes in input_dim as first argument, is size of vocabulary
# the second argument is output_dim is the embedding vector length 
model.add(Embedding(5000,16,input_length=500))
# The LSTM layer takes in as input the number of LSTM units. Other inputs are optional
model.add(LSTM(64,activation='tanh'))
model.add(Dense(1,activation='sigmoid'))

# save the best validation model
model.compile(optimizer=keras.optimizers.Adam(),loss=keras.losses.binary_crossentropy, metrics=['accuracy'])
checkpointer = ModelCheckpoint('best_weights.hdf5', verbose=1, monitor='val_loss', save_best_only=True)

# Train the model    
model.fit(x_train, y_train, batch_size=100, epochs=10, verbose=1, callbacks = [checkpointer], validation_data=(x_val,y_val))

# load in the best fit model
model.load_weights('best_weights.hdf5')
model.compile(optimizer=keras.optimizers.Adam(),loss=keras.losses.binary_crossentropy, metrics=['accuracy'])
score = model.evaluate(x_test, y_test, verbose=0)

print('score', score)