import tensorflow as tf
from keras.applications import Xception
from keras.utils import multi_gpu_model
import os
import numpy as np
from keras.models import Model, load_model
from keras.layers import Input, concatenate, Conv2D, MaxPooling2D, Conv2DTranspose, Cropping2D, Multiply, subtract
from keras.optimizers import Adam
from keras.callbacks import ModelCheckpoint
from keras import backend as K
import h5py

def pixelwise_crossentropy(y_true,y_pred):
    print(y_true.shape)
    class_weights = [1,2]
    y_pred = tf.clip_by_value(y_pred, 10e-8, 1. - 10e-8) #this clip by value command clips the tensor between some min and max values, which are some small amount away from 0 and 1, respectively
    return - tf.reduce_sum(tf.multiply(y_true * tf.log(y_pred),class_weights)) #note that the tf.multiply here is broadcasted multiplication

def dice_coef(y_true, y_pred):
    y_true_f = K.flatten(y_true)
    y_pred_f = K.flatten(y_pred)
    intersection = K.sum(y_true_f * y_pred_f)
    return (2. * intersection) / (K.sum(y_true_f) + K.sum(y_pred_f))


def get_unet(large_tile_size):
    inputs = Input((large_tile_size, large_tile_size, 3))
    conv1 = Conv2D(64, (3, 3), activation='relu', padding='valid')(inputs)
    conv1 = Conv2D(64, (3, 3), activation='relu', padding='valid')(conv1)
    pool1 = MaxPooling2D(pool_size=(2, 2))(conv1)
    
    conv2 = Conv2D(128, (3, 3), activation='relu', padding='valid')(pool1)
    conv2 = Conv2D(128, (3, 3), activation='relu', padding='valid')(conv2)
    pool2 = MaxPooling2D(pool_size=(2, 2))(conv2)
    
    conv3 = Conv2D(256, (3, 3), activation='relu', padding='valid')(pool2)
    conv3 = Conv2D(256, (3, 3), activation='relu', padding='valid')(conv3)
    pool3 = MaxPooling2D(pool_size=(2, 2))(conv3)
    
    conv4 = Conv2D(512, (3, 3), activation='relu', padding='valid')(pool3)
    conv4 = Conv2D(512, (3, 3), activation='relu', padding='valid')(conv4)
    pool4 = MaxPooling2D(pool_size=(2, 2))(conv4)
    
    conv5 = Conv2D(1024, (3, 3), activation='relu', padding='valid')(pool4)
    conv5 = Conv2D(1024, (3, 3), activation='relu', padding='valid')(conv5)
    
    up6 = concatenate([Conv2DTranspose(512, (2, 2), strides=(2, 2), padding='valid')(conv5), Cropping2D(cropping=((4,4),(4,4)))(conv4)], axis=3)
    conv6 = Conv2D(512, (3, 3), activation='relu', padding='valid')(up6)
    conv6 = Conv2D(512, (3, 3), activation='relu', padding='valid')(conv6)
    
    up7 = concatenate([Conv2DTranspose(256, (2, 2), strides=(2, 2), padding='valid')(conv6), Cropping2D(cropping=((16,16),(16,16)))(conv3)], axis=3)
    conv7 = Conv2D(256, (3, 3), activation='relu', padding='valid')(up7)
    conv7 = Conv2D(256, (3, 3), activation='relu', padding='valid')(conv7)
    
    up8 = concatenate([Conv2DTranspose(128, (2, 2), strides=(2, 2), padding='valid')(conv7), Cropping2D(cropping=((40,40),(40,40)))(conv2)], axis=3)
    conv8 = Conv2D(128, (3, 3), activation='relu', padding='valid')(up8)
    conv8 = Conv2D(128, (3, 3), activation='relu', padding='valid')(conv8)
    
    up9 = concatenate([Conv2DTranspose(64, (2, 2), strides=(2, 2), padding='valid')(conv8), Cropping2D(cropping=((88,88),(88,88)))(conv1)], axis=3)
    conv9 = Conv2D(64, (3, 3), activation='relu', padding='valid')(up9)
    conv9 = Conv2D(64, (3, 3), activation='relu', padding='valid')(conv9)
    
    conv10 = Conv2D(2, (1, 1),activation = 'softmax')(conv9) #the output is 388x388x12 containing unscaled log probabilities     print(conv10)
    model = Model(inputs=[inputs], outputs=[conv10])
    
    return model

model = get_unet(572)
model.summary()